#include <stdio.h>
#include "intlist.h"

/* crea una lista vuota e ne restituisce il puntatore radice */
intlist *createlist(void)
{
    return NULL;
}

/* visita una lista e esegue su ogni elemento la funzione op */
void traverse(intlist *p, void (*op)(intlist *))
{
	intlist *q;

	for(q = p; q; q = q->next)
		(*op)(q);
}

/* stampa l'elemento puntato */
void printelem(struct intlist *q)
{
	printf("\t-------\n\t|%5d|\n\t-------\n\t|  %c  |\n\t---%c---\n\t",
			q->dato, q->next ? '.' : 'X', q->next ? '|' : '-');
	if(q->next)
		printf("   |   \n\t   V\n");
}

/* ritorna il numero di elementi nella lista */
int countlist(intlist *p)
{
	int i;
	
	for(i = 0; p; p = p->next, i++);
	return i;
}

/* inserisce un elemento in testa alla lista */
intlist *insert(intlist *p, int elem)
{
	intlist *q = malloc(sizeof(intlist));

	if(!q) {
		fprintf(stderr,"Errore nell'allocazione del nuovo elemento\n");
		exit(-1);	
	};
	q->dato = elem;
	q->next = p;
	return q;
}

/* inserisce un elem nella lista prima del primo elemento >= di elem */ 
intlist *insertinorder(intlist *p, int elem)
{
	intlist *q;
	
	if(!p || p->dato >= elem)
		return insert(p, elem);
	for(q = p; q->next && q->next->dato < elem; q = q->next);
	q->next = insert(q->next, elem);
	return p;
}

/* cancella l'elemento puntato da q dalla lista */
intlist *delete(intlist *p, intlist *q)
{
	intlist *r;

	if(p == q)
		p = p->next;
	else {
		for(r = p; r && r->next != q; r = r->next);
		if(r && r->next == q)
			r->next = r->next->next;
	}
	free(q);
	return p;
}

/* distrugge la lista */
void destroylist(intlist *p)
{
	while(p = delete(p,p));
}

/* concatena la lista con radice q alla lista con radice p */
intlist *listcat(intlist *p, intlist *q)
{
	intlist *r;
	
	if(!p)
		return q;
	for(r = p; r->next; r = r->next);
	r->next = q;
	return p;
}

/* restituisce vero se la funzione check e' vera su almeno un elemento */
char traverseandcheck(intlist *p, char (*check)(intlist *, int), int a)
{
	intlist *q;

	for(q = p; q; q = q->next)
		if((*check)(q,a))
			return 1;
	return 0;
}

/* restituisce il primo elemento per cui check e' vera oppure NULL*/
intlist *getcheckelem(intlist *p, char (*check)(intlist *, int), int a)
{
	intlist *q;

	for(q = p; q; q = q->next)
		if((*check)(q,a))
			return q;
	return NULL;
}

/* restituisce vero se p->dato == elem */ 
char checkexist(intlist *p, int elem)
{
	return p->dato == elem;
}

/* restituisce vero se l'elemento e' presente nella lista */
char existsinlist(intlist *p, int elem)
{
	return traverseandcheck(p,checkexist,elem);
}

/* restituisce il primo elemento q per cui q->dato == elem */
intlist *geteleminlist(intlist *p, int elem)
{
	return getcheckelem(p,checkexist,elem);
}

/* Versioni Ricorsive */

/* ritorna il numero di elementi nella lista: versione ricorsiva */
int rcountlist(intlist *p)
{
    return p ? rcountlist(p->next) + 1 : 0;
}

/* visita una lista e esegue su ogni elemento la funzione op: versione ricorsiva */
void rtraverse(intlist *p, void (*op)(intlist *))
{
    if(p) {
        (*op)(p);
        rtraverse(p->next,op);
    }
}

/* concatena la lista con radice q alla lista con radice p: versione ricorsiva */
intlist *rlistcat(intlist *p, intlist *q)
{
    if(p) 
        p->next = rlistcat(p->next,q);
    return p ? p : q;
}
